@@ -1,45 +1,36 @@
 # -*- coding: utf-8 -*-
-
 from Acquisition import aq_base
+from Acquisition import aq_parent
+from datetime import datetime
+from OFS.interfaces import IFolder
+from OFS.interfaces import ISimpleItem
+from plone.app.layout.navigation.root import getNavigationRootObject
 from Products.CMFCore.interfaces import ISiteRoot
 from Products.CMFCore.utils import getToolByName
-from datetime import datetime
-from plone.app.layout.navigation.root import getNavigationRootObject
+from Products.CMFPlone.utils import get_top_site_from_url
+from z3c.form.interfaces import IForm
+from zope.component import ComponentLookupError
+from zope.component import getMultiAdapter
 from zope.component import providedBy
 from zope.component import queryUtility
 from zope.component.hooks import getSite
+from zope.deprecation import deprecate
+from zope.globalrequest import getRequest
 from zope.i18n import translate
 from zope.i18nmessageid import MessageFactory
 from zope.schema.interfaces import IVocabularyFactory
-from z3c.form.interfaces import IForm
-from zope.component import getMultiAdapter
-from zope.component import ComponentLookupError
+
 import json
-from zope.globalrequest import getRequest
+import zope.deferredimport
+
 
 _ = MessageFactory('plone')
 
 
-try:
-    from plone.app.event import base as pae_base
-    HAS_PAE = True
-except ImportError:
-    HAS_PAE = False
-
-
-def first_weekday():
-    if HAS_PAE:
-        wkday = pae_base.wkday_to_mon1(pae_base.first_weekday())
-        if wkday > 1:
-            return 1  # Default to Monday
-        return wkday
-    else:
-        cal = getToolByName(getSite(), 'portal_calendar', None)
-        if cal:
-            wkday = cal.firstweekday
-            if wkday == 6:  # portal_calendar's Sunday is 6
-                return 0  # Sunday
-        return 1  # other cases: Monday
+zope.deferredimport.deprecated(
+    'Import first_weekday from plone.app.event.base instead',
+    first_weekday='plone.app.event.base:first_weekday',
+)
 
 
 class NotImplemented(Exception):
@@ -68,9 +59,9 @@ def get_date_options(request):
                 _('pickadate_date_format', default='mmmm d, yyyy'),
                 context=request),
             'placeholder': translate(_('Enter date...'), context=request),
-            'today': translate(_(u"Today"), context=request),
-            'clear': translate(_(u"Clear"), context=request),
-        }
+        },
+        'today': translate(_(u"Today"), context=request),
+        'clear': translate(_(u"Clear"), context=request),
     }
 
 
@@ -81,13 +72,14 @@ def get_datetime_options(request):
             _('pickadate_time_format', default='h:i a'),
             context=request),
         'placeholder': translate(_('Enter time...'), context=request),
-        'today': translate(_(u"Today"), context=request),
     }
     return options
 
 
+@deprecate("features were moved into the AjaxSelectWidget, remove in Plone 6")
 def get_ajaxselect_options(context, value, separator, vocabulary_name,
                            vocabulary_view, field_name=None):
+    # code now part of the widget, let it in here for BBB and remove in Plone 6
     options = {'separator': separator}
     if vocabulary_name:
         options['vocabularyUrl'] = '{}/{}?name={}'.format(
@@ -119,38 +111,81 @@ def get_ajaxselect_options(context, value, separator, vocabulary_name,
 
 
 def get_relateditems_options(context, value, separator, vocabulary_name,
-                             vocabulary_view, field_name=None):
-    portal = get_portal()
-    options = get_ajaxselect_options(portal, value, separator,
-                                     vocabulary_name, vocabulary_view,
-                                     field_name)
+                             vocabulary_view, field_name=None,
+                             include_recently_added=True):
+
     if IForm.providedBy(context):
         context = context.context
-    request = getRequest()
-    msgstr = translate(_(u'Search'), context=request)
-    options.setdefault('searchText', msgstr)
-    msgstr = translate(_(u'Entire site'), context=request)
-    options.setdefault('searchAllText', msgstr)
-    msgstr = translate(_('tabs_home',
-                       default=u'Home'),
-                       context=request)
-    options.setdefault('homeText', msgstr)
-    options.setdefault('folderTypes', ['Folder'])
-    options.setdefault(
-        'treeVocabularyUrl',
-        '{}/@@getVocabulary?name=plone.app.vocabularies.Catalog'.format(
-            portal is not None and portal.absolute_url() or '')
-    )
-    options.setdefault('sort_on', 'sortable_title')
-    options.setdefault('sort_order', 'ascending')
 
-    nav_root = getNavigationRootObject(context, portal)
-    options['basePath'] = (
-        '/'.join(nav_root.getPhysicalPath()) if nav_root else '/'
-    )
-    options['rootPath'] = (
-        '/'.join(portal.getPhysicalPath()) if portal else '/'
+    request = getRequest()
+    site = get_top_site_from_url(context, request)
+    options = {
+        'separator': separator,
+    }
+    if not vocabulary_name:
+        # we need a vocabulary!
+        raise ValueError('RelatedItems needs a vocabulary')
+    options['vocabularyUrl'] = '{0}/{1}?name={2}'.format(
+        get_context_url(site), vocabulary_view, vocabulary_name,
     )
+    if field_name:
+        options['vocabularyUrl'] += '&field={0}'.format(field_name)
+    if value:
+        options['initialValues'] = {}
+        catalog = False
+        if vocabulary_name == 'plone.app.vocabularies.Catalog':
+            catalog = getToolByName(getSite(), 'portal_catalog')
+        for value in value.split(separator):
+            title = value
+            if catalog:
+                result = catalog(UID=value)
+                title = result[0].Title if result else value
+            options['initialValues'][value] = title
+
+    nav_root = getNavigationRootObject(context, site)
+
+    if not ISimpleItem.providedBy(context):
+        context = nav_root
+
+    # basePath - start to search/browse in here.
+    base_path_context = context
+    if not IFolder.providedBy(base_path_context):
+        base_path_context = aq_parent(base_path_context)
+    if not base_path_context:
+        base_path_context = nav_root
+    options['basePath'] = '/'.join(base_path_context.getPhysicalPath())
+
+    # rootPath - Only display breadcrumb elements deeper than this path.
+    options['rootPath'] = '/'.join(site.getPhysicalPath()) if site else '/'
+
+    # rootUrl: Visible URL up to the rootPath. This is prepended to the
+    # currentPath to generate submission URLs.
+    options['rootUrl'] = site.absolute_url() if site else ''
+
+    # contextPath - current edited object. Will not be available to select.
+    options['contextPath'] = '/'.join(context.getPhysicalPath())
+
+    if base_path_context != nav_root:
+        options['favorites'] = [
+            {
+                'title': _(u'Current Content'),
+                'path': '/'.join(base_path_context.getPhysicalPath())
+            }, {
+                'title': _(u'Start Page'),
+                'path': '/'.join(nav_root.getPhysicalPath())
+            }
+        ]
+
+    if include_recently_added:
+        # Options for recently used key
+        tool = getToolByName(context, 'portal_membership')
+        user = tool.getAuthenticatedMember()
+        options['recentlyUsed'] = False  # Keep that off in Plone 5.1
+        options['recentlyUsedKey'] = (u'relateditems_recentlyused_%s_%s' % (
+            field_name or '',
+            user.id
+        ))  # use string substitution with %s here for automatic str casting.
+
     return options
 
 
@@ -163,7 +198,18 @@ def get_querystring_options(context, querystring_view):
     return {
         'indexOptionsUrl': '{}/{}'.format(portal_url, querystring_view),
         'previewURL': '%s/@@querybuilder_html_results' % base_url,
-        'previewCountURL': '%s/@@querybuildernumberofresults' % base_url
+        'previewCountURL': '%s/@@querybuildernumberofresults' % base_url,
+        'patternDateOptions': get_date_options(getRequest()),
+        'patternAjaxSelectOptions': {'separator': ';'},
+        'patternRelateditemsOptions': get_relateditems_options(
+            context,
+            None,
+            ';',
+            'plone.app.vocabularies.Catalog',
+            '@@getVocabulary',
+            'relatedItems',
+            include_recently_added=False
+        )
     }
 
 
